@@ -37,6 +37,7 @@ class Transport:
         self.operation_timeout = operation_timeout
         self.logger = logging.getLogger(__name__)
 
+        self._close_session = not session
         self.session = session or requests.Session()
         self.session.mount("file://", FileAdapter())
         self.session.headers["User-Agent"] = "Zeep/%s (www.python-zeep.org)" % (
@@ -154,6 +155,10 @@ class Transport:
         yield
         self.operation_timeout = old_timeout
 
+    def __del__(self):
+        if self._close_session:
+            self.session.close()
+
 
 class AsyncTransport(Transport):
     """Asynchronous Transport class using httpx.
@@ -170,13 +175,13 @@ class AsyncTransport(Transport):
         cache=None,
         timeout=300,
         operation_timeout=None,
-        session=None,
         verify_ssl=True,
         proxy=None,
     ):
         if httpx is None:
             raise RuntimeError("The AsyncTransport is based on the httpx module")
 
+        self._close_session = False
         self.cache = cache
         self.wsdl_client = wsdl_client or httpx.Client(
             verify=verify_ssl,
@@ -214,7 +219,7 @@ class AsyncTransport(Transport):
         self.logger.debug("HTTP Post to %s:\n%s", address, message)
         response = await self.client.post(
             address,
-            data=message,
+            content=message,
             headers=headers,
         )
         self.logger.debug(
