@@ -5,8 +5,13 @@ See https://www.w3.org/TR/SOAP-attachments
 """
 
 import base64
+import sys
+
+if sys.version_info >= (3, 8):
+    from functools import cached_property
+else:
+    from cached_property import cached_property
 
-from cached_property import cached_property
 from requests.structures import CaseInsensitiveDict
 
 
