@@ -1,30 +1,63 @@
+from __future__ import annotations
+
 import copy
 import logging
+import sys
+import typing
 from collections import OrderedDict, deque
 from itertools import chain
+from typing import Optional
 
-from cached_property import threaded_cached_property
+if sys.version_info >= (3, 8):
+    from functools import cached_property as threaded_cached_property
+else:
+    from cached_property import threaded_cached_property
+
+from lxml import etree
 
 from zeep.exceptions import UnexpectedElementError, XMLParseError
-from zeep.xsd.const import NotSet, SkipValue, xsi_ns
+from zeep.xsd.const import Nil, NotSet, SkipValue, xsi_ns
+from zeep.xsd.context import XmlParserContext
 from zeep.xsd.elements import (
-    Any, AnyAttribute, AttributeGroup, Choice, Element, Group, Sequence)
+    Any,
+    AnyAttribute,
+    AttributeGroup,
+    Choice,
+    Element,
+    Group,
+    Sequence,
+)
 from zeep.xsd.elements.indicators import OrderIndicator
 from zeep.xsd.types.any import AnyType
 from zeep.xsd.types.simple import AnySimpleType
 from zeep.xsd.utils import NamePrefixGenerator
 from zeep.xsd.valueobjects import ArrayValue, CompoundValue
 
+if typing.TYPE_CHECKING:
+    from zeep.xsd.schema import Schema
+    from zeep.xsd.types.base import Type
+else:
+    Schema = Type = None
+
 logger = logging.getLogger(__name__)
 
-__all__ = ['ComplexType']
+__all__ = ["ComplexType"]
+# Recursive alias
+_ObjectList = typing.List[typing.Union[CompoundValue, None, "_ObjectList"]]
 
 
 class ComplexType(AnyType):
-    _xsd_name = None
+    _xsd_name: typing.Optional[str] = None
 
-    def __init__(self, element=None, attributes=None,
-                 restriction=None, extension=None, qname=None, is_global=False):
+    def __init__(
+        self,
+        element=None,
+        attributes=None,
+        restriction=None,
+        extension=None,
+        qname=None,
+        is_global: bool = False,
+    ):
         if element and type(element) == list:
             element = Sequence(element)
 
@@ -33,8 +66,8 @@ class ComplexType(AnyType):
         self._attributes = attributes or []
         self._restriction = restriction
         self._extension = extension
-        self._extension_types = tuple()
-        super(ComplexType, self).__init__(qname=qname, is_global=is_global)
+        self._extension_types: typing.List[typing.Type] = []
+        super().__init__(qname=qname, is_global=is_global)
 
     def __call__(self, *args, **kwargs):
         if self._array_type:
@@ -42,35 +75,39 @@ class ComplexType(AnyType):
         return self._value_class(*args, **kwargs)
 
     @property
-    def accepted_types(self):
-        return (self._value_class,) + self._extension_types
+    def accepted_types(self) -> typing.List[typing.Type]:
+        return [self._value_class] + self._extension_types
 
     @threaded_cached_property
-    def _array_class(self):
+    def _array_class(self) -> typing.Type[ArrayValue]:
         assert self._array_type
         return type(
-            self.__class__.__name__, (ArrayValue,),
-            {'_xsd_type': self, '__module__': 'zeep.objects'})
+            self.__class__.__name__,
+            (ArrayValue,),
+            {"_xsd_type": self, "__module__": "zeep.objects"},
+        )
 
     @threaded_cached_property
-    def _value_class(self):
+    def _value_class(self) -> typing.Type[CompoundValue]:
         return type(
-            self.__class__.__name__, (CompoundValue,),
-            {'_xsd_type': self, '__module__': 'zeep.objects'})
+            self.__class__.__name__,
+            (CompoundValue,),
+            {"_xsd_type": self, "__module__": "zeep.objects"},
+        )
 
     def __str__(self):
-        return '%s(%s)' % (self.__class__.__name__, self.signature())
+        return "%s(%s)" % (self.__class__.__name__, self.signature())
 
     @threaded_cached_property
     def attributes(self):
-        generator = NamePrefixGenerator(prefix='_attr_')
+        generator = NamePrefixGenerator(prefix="_attr_")
         result = []
         elm_names = {name for name, elm in self.elements if name is not None}
         for attr in self._attributes_unwrapped:
             if attr.name is None:
                 name = generator.get_name()
             elif attr.name in elm_names:
-                name = 'attr__%s' % attr.name
+                name = "attr__%s" % attr.name
             else:
                 name = attr.name
             result.append((name, attr))
@@ -107,9 +144,19 @@ class ComplexType(AnyType):
         if self._array_type:
             name = generator.get_name()
             if isinstance(self._element, Group):
-                result = [(name, Sequence([
-                    Any(max_occurs='unbounded', restrict=self._array_type.array_type)
-                ]))]
+                result = [
+                    (
+                        name,
+                        Sequence(
+                            [
+                                Any(
+                                    max_occurs="unbounded",
+                                    restrict=self._array_type.array_type,
+                                )
+                            ]
+                        ),
+                    )
+                ]
             else:
                 result = [(name, self._element)]
         else:
@@ -121,24 +168,24 @@ class ComplexType(AnyType):
     @property
     def _array_type(self):
         attrs = {attr.qname.text: attr for attr in self._attributes if attr.qname}
-        array_type = attrs.get('{http://schemas.xmlsoap.org/soap/encoding/}arrayType')
+        array_type = attrs.get("{http://schemas.xmlsoap.org/soap/encoding/}arrayType")
         return array_type
 
-    def parse_xmlelement(self, xmlelement, schema=None, allow_none=True,
-                         context=None, schema_type=None):
+    def parse_xmlelement(
+        self,
+        xmlelement: etree._Element,
+        schema: Optional[Schema] = None,
+        allow_none: bool = True,
+        context: XmlParserContext = None,
+        schema_type: Optional[Type] = None,
+    ) -> typing.Optional[typing.Union[str, CompoundValue, typing.List[etree._Element]]]:
         """Consume matching xmlelements and call parse() on each
 
         :param xmlelement: XML element objects
-        :type xmlelement: lxml.etree._Element
         :param schema: The parent XML schema
-        :type schema: zeep.xsd.Schema
         :param allow_none: Allow none
-        :type allow_none: bool
         :param context: Optional parsing context (for inline schemas)
-        :type context: zeep.xsd.context.XmlParserContext
         :param schema_type: The original type (not overriden via xsi:type)
-        :type schema_type: zeep.xsd.types.base.Type
-        :rtype: dict or None
 
         """
         # If this is an empty complexType (<xsd:complexType name="x"/>)
@@ -151,14 +198,17 @@ class ComplexType(AnyType):
         # If this complexType extends a simpleType then we have no nested
         # elements. Parse it directly via the type object. This is the case
         # for xsd:simpleContent
-        if isinstance(self._element, Element) and isinstance(self._element.type, AnySimpleType):
+        if isinstance(self._element, Element) and isinstance(
+            self._element.type, AnySimpleType
+        ):
             name, element = self.elements_nested[0]
             init_kwargs[name] = element.type.parse_xmlelement(
-                xmlelement, schema, name, context=context)
+                xmlelement, schema, name, context=context
+            )
         else:
             elements = deque(xmlelement.iterchildren())
             if allow_none and len(elements) == 0 and len(attributes) == 0:
-                return
+                return None
 
             # Parse elements. These are always indicator elements (all, choice,
             # group, sequence)
@@ -166,7 +216,8 @@ class ComplexType(AnyType):
             for name, element in self.elements_nested:
                 try:
                     result = element.parse_xmlelements(
-                        elements, schema, name, context=context)
+                        elements, schema, name, context=context
+                    )
                     if result:
                         init_kwargs.update(result)
                 except UnexpectedElementError as exc:
@@ -174,10 +225,10 @@ class ComplexType(AnyType):
 
             # Check if all children are consumed (parsed)
             if elements:
-                if schema.strict:
+                if schema and schema.settings.strict:
                     raise XMLParseError("Unexpected element %r" % elements[0].tag)
                 else:
-                    init_kwargs['_raw_elements'] = elements
+                    init_kwargs["_raw_elements"] = elements
 
         # Parse attributes
         if attributes:
@@ -185,24 +236,27 @@ class ComplexType(AnyType):
             for name, attribute in self.attributes:
                 if attribute.name:
                     if attribute.qname.text in attributes:
-                        value = attributes.pop(attribute.qname.text)
-                        init_kwargs[name] = attribute.parse(value)
+                        attr_value = attributes.pop(attribute.qname.text)
+                        init_kwargs[name] = attribute.parse(attr_value)
                 else:
                     init_kwargs[name] = attribute.parse(attributes)
 
-        value = self._value_class(**init_kwargs)
+        value: CompoundValue = self._value_class(**init_kwargs)
         schema_type = schema_type or self
-        if schema_type and getattr(schema_type, '_array_type', None):
-            value = schema_type._array_class.from_value_object(value)
+        if schema_type and getattr(schema_type, "_array_type", None):
+            return schema_type._array_class.from_value_object(value)
         return value
 
-    def render(self, parent, value, xsd_type=None, render_path=None):
-        """Serialize the given value lxml.Element subelements on the parent
+    def render(
+        self,
+        node: etree._Element,
+        value: typing.Union[list, dict, CompoundValue],
+        xsd_type: "ComplexType" = None,
+        render_path=None,
+    ) -> None:
+        """Serialize the given value lxml.Element subelements on the node
         element.
 
-        :type parent: lxml.etree._Element
-        :type value: Union[list, dict, zeep.xsd.valueobjects.CompoundValue]
-        :type xsd_type: zeep.xsd.types.base.Type
         :param render_path: list
 
         """
@@ -223,15 +277,15 @@ class ComplexType(AnyType):
         for name, attribute in self.attributes:
             attr_value = value[name] if name in value else NotSet
             child_path = render_path + [name]
-            attribute.render(parent, attr_value, child_path)
+            attribute.render(node, attr_value, child_path)
 
         if (
             len(self.elements_nested) == 1
-            and isinstance(value, self.accepted_types)
+            and isinstance(value, tuple(self.accepted_types))
             and not isinstance(value, (list, dict, CompoundValue))
         ):
             element = self.elements_nested[0][1]
-            element.type.render(parent, value, None, child_path)
+            element.type.render(node, value, None, child_path)
             return
 
         # Render sub elements
@@ -248,28 +302,29 @@ class ComplexType(AnyType):
                 continue
 
             if isinstance(element, Element):
-                element.type.render(parent, element_value, None, child_path)
+                element.type.render(node, element_value, None, child_path)
             else:
-                element.render(parent, element_value, child_path)
+                element.render(node, element_value, child_path)
 
         if xsd_type:
             if xsd_type._xsd_name:
-                parent.set(xsi_ns('type'), xsd_type._xsd_name)
+                node.set(xsi_ns("type"), xsd_type._xsd_name)
             if xsd_type.qname:
-                parent.set(xsi_ns('type'), xsd_type.qname)
+                node.set(xsi_ns("type"), xsd_type.qname)
 
-    def parse_kwargs(self, kwargs, name, available_kwargs):
+    def parse_kwargs(
+        self,
+        kwargs: typing.Dict[str, typing.Any],
+        name: str,
+        available_kwargs: typing.Set[str],
+    ) -> typing.Dict[str, typing.Any]:
         """Parse the kwargs for this type and return the accepted data as
         a dict.
 
         :param kwargs: The kwargs
-        :type kwargs: dict
         :param name: The name as which this type is registered in the parent
-        :type name: str
         :param available_kwargs: The kwargs keys which are still available,
          modified in place
-        :type available_kwargs: set
-        :rtype: dict
 
         """
         value = None
@@ -279,11 +334,15 @@ class ComplexType(AnyType):
             value = kwargs[name]
             available_kwargs.remove(name)
 
-            value = self._create_object(value, name)
+            if value is not Nil:
+                value = self._create_object(value, name)
+
             return {name: value}
         return {}
 
-    def _create_object(self, value, name):
+    def _create_object(
+        self, value: typing.Union[list, dict, CompoundValue, None], name: str
+    ) -> typing.Union[CompoundValue, None, _ObjectList]:
         """Return the value as a CompoundValue object
 
         :type value: str
@@ -360,7 +419,7 @@ class ComplexType(AnyType):
             new_attributes = OrderedDict()
             for attr in attributes:
                 if isinstance(attr, AnyAttribute):
-                    new_attributes['##any'] = attr
+                    new_attributes["##any"] = attr
                 else:
                     new_attributes[attr.qname.text] = attr
             attributes = new_attributes.values()
@@ -375,7 +434,9 @@ class ComplexType(AnyType):
 
             element = self._element.clone(self._element.name)
             if isinstance(base_element, OrderIndicator):
-                if isinstance(self._element, Choice):
+                if isinstance(base_element, Choice):
+                    element.insert(0, base_element)
+                elif isinstance(self._element, Choice):
                     element = base_element.clone(self._element.name)
                     element.append(self._element)
                 elif isinstance(element, OrderIndicator):
@@ -386,20 +447,21 @@ class ComplexType(AnyType):
                         element.child.insert(0, item)
 
             elif isinstance(self._element, Group):
-                raise NotImplementedError('TODO')
+                raise NotImplementedError("TODO")
             else:
                 pass  # Element (ignore for now)
 
         elif self._element or base_element:
             element = self._element or base_element
         else:
-            element = Element('_value_1', base)
+            element = Element("_value_1", base)
 
         new = self.__class__(
             element=element,
             attributes=attributes,
             qname=self.qname,
-            is_global=self.is_global)
+            is_global=self.is_global,
+        )
 
         new._extension_types = base.accepted_types
         return new
@@ -415,15 +477,14 @@ class ComplexType(AnyType):
 
 
         """
-        attributes = list(
-            chain(base._attributes_unwrapped, self._attributes_unwrapped))
+        attributes = list(chain(base._attributes_unwrapped, self._attributes_unwrapped))
 
         # Make sure we don't have duplicate (self is leading)
         if base._attributes_unwrapped and self._attributes_unwrapped:
             new_attributes = OrderedDict()
             for attr in attributes:
                 if isinstance(attr, AnyAttribute):
-                    new_attributes['##any'] = attr
+                    new_attributes["##any"] = attr
                 else:
                     new_attributes[attr.qname.text] = attr
             attributes = list(new_attributes.values())
@@ -435,7 +496,8 @@ class ComplexType(AnyType):
             element=self._element or base._element,
             attributes=attributes,
             qname=self.qname,
-            is_global=self.is_global)
+            is_global=self.is_global,
+        )
         return new.resolve()
 
     def signature(self, schema=None, standalone=True):
@@ -445,11 +507,11 @@ class ComplexType(AnyType):
             parts.append(part)
 
         for name, attribute in self.attributes:
-            part = '%s: %s' % (name, attribute.signature(schema, standalone=False))
+            part = "%s: %s" % (name, attribute.signature(schema, standalone=False))
             parts.append(part)
 
-        value = ', '.join(parts)
+        value = ", ".join(parts)
         if standalone:
-            return '%s(%s)' % (self.get_prefixed_name(schema), value)
+            return "%s(%s)" % (self.get_prefixed_name(schema), value)
         else:
             return value
