@@ -6,6 +6,8 @@ import requests_mock
 from tests.utils import load_xml
 from zeep import client, xsd
 from zeep.exceptions import Error
+from zeep.transports import Transport
+from zeep.wsdl import Document
 
 
 def test_bind():
@@ -14,6 +16,13 @@ def test_bind():
     assert service
 
 
+def test_bind_existing_document():
+    wsdl = Document("tests/wsdl_files/soap.wsdl", transport=Transport())
+    client_obj = client.Client(wsdl)
+    service = client_obj.bind()
+    assert service
+
+
 def test_unknown_transport():
     client_obj = client.Client("tests/wsdl_files/soap_transport_err.wsdl")
     service = client_obj.bind()
@@ -43,6 +52,11 @@ def test_service_proxy_non_existing():
         assert client_obj.service.NonExisting
 
 
+def test_context_manager():
+    with client.Client("tests/wsdl_files/soap.wsdl") as c:
+        assert c
+
+
 def test_service_proxy_dir_operations():
     client_obj = client.Client("tests/wsdl_files/soap.wsdl")
     operations = [op for op in dir(client_obj.service) if not op.startswith("_")]
