@@ -11,23 +11,23 @@ def test_huge_text():
     # libxml2>=2.7.3 has XML_MAX_TEXT_LENGTH 10000000 without XML_PARSE_HUGE
     settings = Settings(xml_huge_tree=True)
     tree = parse_xml(
-        u"""
+        """
         <s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
          <s:Body>
           <HugeText xmlns="http://hugetext">%s</HugeText>
          </s:Body>
         </s:Envelope>
     """
-        % (u"\u00e5" * 10000001),
+        % ("\u00e5" * 10000001),
         DummyTransport(),
         settings=settings,
     )
 
-    assert tree[0][0].text == u"\u00e5" * 10000001
+    assert tree[0][0].text == "\u00e5" * 10000001
 
 
 def test_allow_entities_and_dtd():
-    xml = u"""
+    xml = """
         <!DOCTYPE Author [
           <!ENTITY writer "Donald Duck.">
         ]>
